@@ -16,6 +16,7 @@
 #include <QPalette>
 #include <QQuickWindow>
 #include <QStandardPaths>
+#include <QTimeZone>
 
 using namespace KOSMIndoorMap;
 
@@ -135,9 +136,14 @@ void MapItem::loaderDone()
     m_sg.clear();
 
     if (!m_loader->hasError()) {
-        m_data = m_loader->takeData();
+        auto data = m_loader->takeData();
+        if (data.regionCode().isEmpty()) {
+            data.setRegionCode(m_data.regionCode());
+        }
+        data.setTimeZone(m_data.timeZone());
+        m_data = std::move(data);
         m_view->setSceneBoundingBox(m_data.boundingBox());
-        m_controller.setDataSet(&m_data);
+        m_controller.setMapData(m_data);
         m_style.compile(m_data.dataSet());
         m_controller.setStyleSheet(&m_style);
         m_view->setLevel(0);
@@ -171,8 +177,8 @@ void MapItem::clear()
     }
 
     m_sg.clear();
-    m_controller.setDataSet(nullptr);
     m_data = MapData();
+    m_controller.setMapData(m_data);
     emit mapDataChanged();
     emit errorChanged();
     update();
@@ -200,7 +206,9 @@ QVariant MapItem::overlaySources() const
 
 void MapItem::setOverlaySources(const QVariant &overlays)
 {
-    std::vector<OverlaySource> sources;
+    const auto oldOwnedOverlays = std::move(m_ownedOverlaySources);
+
+    std::vector<QPointer<AbstractOverlaySource>> sources;
     if (overlays.canConvert<QVariantList>()) {
         const auto l = overlays.value<QVariantList>();
         for (const auto &v : l) {
@@ -210,25 +218,68 @@ void MapItem::setOverlaySources(const QVariant &overlays)
         addOverlaySource(sources, overlays);
     }
 
+    for (const auto &overlay : sources) {
+        connect(overlay.data(), &AbstractOverlaySource::update, this, &MapItem::overlayUpdate, Qt::UniqueConnection);
+        connect(overlay.data(), &AbstractOverlaySource::reset, this, &MapItem::overlayReset, Qt::UniqueConnection);
+    }
+
     m_controller.setOverlaySources(std::move(sources));
     emit overlaySourcesChanged();
     update();
 }
 
-void MapItem::addOverlaySource(std::vector<OverlaySource> &overlaySources, const QVariant &source)
+void MapItem::addOverlaySource(std::vector<QPointer<AbstractOverlaySource>> &overlaySources, const QVariant &source)
 {
     const auto obj = source.value<QObject*>();
     if (auto model = qobject_cast<QAbstractItemModel*>(obj)) {
-        OverlaySource overlay(model);
-        overlay.setUpdateCallback(this, [this]() {
-            m_controller.overlaySourceUpdated();
-            update();
-        });
-        overlay.setResetCallback(this, [this]() {
-            m_style.compile(m_data.dataSet());
-        });
-        overlaySources.push_back(std::move(overlay));
+        auto overlay = std::make_unique<ModelOverlaySource>(model);
+        overlaySources.push_back(overlay.get());
+        m_ownedOverlaySources.push_back(std::move(overlay));
+    } else if (auto source = qobject_cast<AbstractOverlaySource*>(obj)) {
+        overlaySources.push_back(source);
     } else {
         qWarning() << "unsupported overlay source:" << source << obj;
     }
 }
+
+void MapItem::overlayUpdate()
+{
+    m_controller.overlaySourceUpdated();
+    update();
+}
+
+void MapItem::overlayReset()
+{
+    m_style.compile(m_data.dataSet());
+}
+
+QString MapItem::region() const
+{
+    return m_data.regionCode();
+}
+
+void MapItem::setRegion(const QString &region)
+{
+    if (m_data.regionCode() == region) {
+        return;
+    }
+
+    m_data.setRegionCode(region);
+    emit regionChanged();
+}
+
+QString MapItem::timeZoneId() const
+{
+    return QString::fromUtf8(m_data.timeZone().id());
+}
+
+void MapItem::setTimeZoneId(const QString &tz)
+{
+    const auto tzId = tz.toUtf8();
+    if (m_data.timeZone().id() == tzId) {
+        return;
+    }
+
+    m_data.setTimeZone(QTimeZone(tzId));
+    emit timeZoneChanged();
+}
