@@ -6,6 +6,7 @@
 
 #include "mapcssselector_p.h"
 #include "mapcsscondition_p.h"
+#include "mapcssresult_p.h"
 #include "mapcssstate_p.h"
 
 #include <QDebug>
@@ -31,7 +32,7 @@ void MapCSSBasicSelector::compile(const OSM::DataSet &dataSet)
     }
 }
 
-bool MapCSSBasicSelector::matches(const MapCSSState &state) const
+bool MapCSSBasicSelector::matches(const MapCSSState &state, const MapCSSResult &result) const
 {
     // check zoom conditions first, as this is the cheapest one and can avoid expensive tag lookups we it doesn't match
     if (m_zoomLow > 0 && state.zoomLevel < m_zoomLow) {
@@ -71,7 +72,11 @@ bool MapCSSBasicSelector::matches(const MapCSSState &state) const
         case Any: break;
     }
 
-    return std::all_of(conditions.begin(), conditions.end(), [state](const auto &cond) { return cond->matches(state); });
+    if (!m_class.isEmpty() && !result.hasClass(m_class)) {
+        return false;
+    }
+
+    return std::all_of(conditions.begin(), conditions.end(), [&state](const auto &cond) { return cond->matches(state); });
 }
 
 bool MapCSSBasicSelector::matchesCanvas(const MapCSSState &state) const
@@ -87,7 +92,7 @@ bool MapCSSBasicSelector::matchesCanvas(const MapCSSState &state) const
         return false;
     }
 
-    return std::all_of(conditions.begin(), conditions.end(), [state](const auto &cond) { return cond->matchesCanvas(state); });
+    return std::all_of(conditions.begin(), conditions.end(), [&state](const auto &cond) { return cond->matchesCanvas(state); });
 }
 
 struct {
@@ -177,9 +182,10 @@ void MapCSSChainedSelector::compile(const OSM::DataSet &dataSet)
     }
 }
 
-bool MapCSSChainedSelector::matches(const MapCSSState &state) const
+bool MapCSSChainedSelector::matches(const MapCSSState &state, const MapCSSResult &result) const
 {
     Q_UNUSED(state); // TODO
+    Q_UNUSED(result);
     return false;
 }
 
@@ -210,14 +216,14 @@ void MapCSSUnionSelector::compile(const OSM::DataSet &dataSet)
     }
 }
 
-bool MapCSSUnionSelector::matches(const MapCSSState &state) const
+bool MapCSSUnionSelector::matches(const MapCSSState &state, const MapCSSResult &result) const
 {
-    return std::any_of(selectors.begin(), selectors.end(), [state](const auto &selector) { return selector->matches(state); });
+    return std::any_of(selectors.begin(), selectors.end(), [&state, &result](const auto &selector) { return selector->matches(state, result); });
 }
 
 bool MapCSSUnionSelector::matchesCanvas(const MapCSSState &state) const
 {
-    return std::any_of(selectors.begin(), selectors.end(), [state](const auto &selector) { return selector->matchesCanvas(state); });
+    return std::any_of(selectors.begin(), selectors.end(), [&state](const auto &selector) { return selector->matchesCanvas(state); });
 }
 
 void MapCSSUnionSelector::write(QIODevice *out) const
